package com.sun.jndi.toolkit.ctx;

import javax.naming.InvalidNameException;
import java.util.Enumeration;
import javax.naming.CannotProceedException;
import javax.naming.spi.NamingManager;
import java.util.Hashtable;
import javax.naming.NameParser;
import javax.naming.Binding;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.spi.ResolveResult;
import javax.naming.Name;
import javax.naming.CompositeName;
import javax.naming.spi.Resolver;
import javax.naming.Context;

public abstract class PartialCompositeContext implements Context, Resolver
{
    protected static final int _PARTIAL = 1;
    protected static final int _COMPONENT = 2;
    protected static final int _ATOMIC = 3;
    protected int _contextType;
    static final CompositeName _EMPTY_NAME;
    static CompositeName _NNS_NAME;
    
    protected PartialCompositeContext() {
        this._contextType = 1;
    }
    
    protected abstract ResolveResult p_resolveToClass(Name p0, Class<?> p1, Continuation p2) throws NamingException;
    
    protected abstract Object p_lookup(Name p0, Continuation p1) throws NamingException;
    
    protected abstract Object p_lookupLink(Name p0, Continuation p1) throws NamingException;
    
    protected abstract NamingEnumeration<NameClassPair> p_list(Name p0, Continuation p1) throws NamingException;
    
    protected abstract NamingEnumeration<Binding> p_listBindings(Name p0, Continuation p1) throws NamingException;
    
    protected abstract void p_bind(Name p0, Object p1, Continuation p2) throws NamingException;
    
    protected abstract void p_rebind(Name p0, Object p1, Continuation p2) throws NamingException;
    
    protected abstract void p_unbind(Name p0, Continuation p1) throws NamingException;
    
    protected abstract void p_destroySubcontext(Name p0, Continuation p1) throws NamingException;
    
    protected abstract Context p_createSubcontext(Name p0, Continuation p1) throws NamingException;
    
    protected abstract void p_rename(Name p0, Name p1, Continuation p2) throws NamingException;
    
    protected abstract NameParser p_getNameParser(Name p0, Continuation p1) throws NamingException;
    
    protected Hashtable<?, ?> p_getEnvironment() throws NamingException {
        return this.getEnvironment();
    }
    
    @Override
    public ResolveResult resolveToClass(String n, Class<? extends Context> clazz) throws NamingException {
        return this.resolveToClass(new CompositeName(n), clazz);
    }
    
    @Override
    public ResolveResult resolveToClass(Name name, Class<? extends Context> clazz) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        ResolveResult resolveResult;
        try {
            resolveResult = this.p_resolveToClass(name, clazz, continuation);
            while (continuation.isContinue()) {
                resolveResult = getPCContext(continuation).p_resolveToClass(continuation.getRemainingName(), clazz, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            Context continuationContext = NamingManager.getContinuationContext(cpe);
            if (!(continuationContext instanceof Resolver)) {
                throw cpe;
            }
            resolveResult = ((Resolver)continuationContext).resolveToClass(cpe.getRemainingName(), clazz);
        }
        return resolveResult;
    }
    
    @Override
    public Object lookup(String n) throws NamingException {
        return this.lookup(new CompositeName(n));
    }
    
    @Override
    public Object lookup(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        Object o;
        try {
            o = this.p_lookup(name, continuation);
            while (continuation.isContinue()) {
                o = getPCContext(continuation).p_lookup(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            o = NamingManager.getContinuationContext(cpe).lookup(cpe.getRemainingName());
        }
        return o;
    }
    
    @Override
    public void bind(String n, Object o) throws NamingException {
        this.bind(new CompositeName(n), o);
    }
    
    @Override
    public void bind(Name name, Object o) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_bind(name, o, continuation);
            while (continuation.isContinue()) {
                getPCContext(continuation).p_bind(continuation.getRemainingName(), o, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            NamingManager.getContinuationContext(cpe).bind(cpe.getRemainingName(), o);
        }
    }
    
    @Override
    public void rebind(String n, Object o) throws NamingException {
        this.rebind(new CompositeName(n), o);
    }
    
    @Override
    public void rebind(Name name, Object o) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_rebind(name, o, continuation);
            while (continuation.isContinue()) {
                getPCContext(continuation).p_rebind(continuation.getRemainingName(), o, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            NamingManager.getContinuationContext(cpe).rebind(cpe.getRemainingName(), o);
        }
    }
    
    @Override
    public void unbind(String n) throws NamingException {
        this.unbind(new CompositeName(n));
    }
    
    @Override
    public void unbind(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_unbind(name, continuation);
            while (continuation.isContinue()) {
                getPCContext(continuation).p_unbind(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            NamingManager.getContinuationContext(cpe).unbind(cpe.getRemainingName());
        }
    }
    
    @Override
    public void rename(String n, String n2) throws NamingException {
        this.rename(new CompositeName(n), new CompositeName(n2));
    }
    
    @Override
    public void rename(Name name, Name remainingNewName) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_rename(name, remainingNewName, continuation);
            while (continuation.isContinue()) {
                getPCContext(continuation).p_rename(continuation.getRemainingName(), remainingNewName, continuation);
            }
        }
        catch (CannotProceedException cpe) {
            Context continuationContext = NamingManager.getContinuationContext(cpe);
            if (cpe.getRemainingNewName() != null) {
                remainingNewName = cpe.getRemainingNewName();
            }
            continuationContext.rename(cpe.getRemainingName(), remainingNewName);
        }
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(String n) throws NamingException {
        return this.list(new CompositeName(n));
    }
    
    @Override
    public NamingEnumeration<NameClassPair> list(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        NamingEnumeration<NameClassPair> namingEnumeration;
        try {
            namingEnumeration = this.p_list(name, continuation);
            while (continuation.isContinue()) {
                namingEnumeration = getPCContext(continuation).p_list(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            namingEnumeration = NamingManager.getContinuationContext(cpe).list(cpe.getRemainingName());
        }
        return namingEnumeration;
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(String n) throws NamingException {
        return this.listBindings(new CompositeName(n));
    }
    
    @Override
    public NamingEnumeration<Binding> listBindings(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        NamingEnumeration<Binding> namingEnumeration;
        try {
            namingEnumeration = this.p_listBindings(name, continuation);
            while (continuation.isContinue()) {
                namingEnumeration = getPCContext(continuation).p_listBindings(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            namingEnumeration = NamingManager.getContinuationContext(cpe).listBindings(cpe.getRemainingName());
        }
        return namingEnumeration;
    }
    
    @Override
    public void destroySubcontext(String n) throws NamingException {
        this.destroySubcontext(new CompositeName(n));
    }
    
    @Override
    public void destroySubcontext(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        try {
            this.p_destroySubcontext(name, continuation);
            while (continuation.isContinue()) {
                getPCContext(continuation).p_destroySubcontext(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            NamingManager.getContinuationContext(cpe).destroySubcontext(cpe.getRemainingName());
        }
    }
    
    @Override
    public Context createSubcontext(String n) throws NamingException {
        return this.createSubcontext(new CompositeName(n));
    }
    
    @Override
    public Context createSubcontext(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        Context context;
        try {
            context = this.p_createSubcontext(name, continuation);
            while (continuation.isContinue()) {
                context = getPCContext(continuation).p_createSubcontext(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            context = NamingManager.getContinuationContext(cpe).createSubcontext(cpe.getRemainingName());
        }
        return context;
    }
    
    @Override
    public Object lookupLink(String n) throws NamingException {
        return this.lookupLink(new CompositeName(n));
    }
    
    @Override
    public Object lookupLink(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        Object o;
        try {
            o = this.p_lookupLink(name, continuation);
            while (continuation.isContinue()) {
                o = getPCContext(continuation).p_lookupLink(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            o = NamingManager.getContinuationContext(cpe).lookupLink(cpe.getRemainingName());
        }
        return o;
    }
    
    @Override
    public NameParser getNameParser(String n) throws NamingException {
        return this.getNameParser(new CompositeName(n));
    }
    
    @Override
    public NameParser getNameParser(Name name) throws NamingException {
        Continuation continuation = new Continuation(name, this.p_getEnvironment());
        NameParser nameParser;
        try {
            nameParser = this.p_getNameParser(name, continuation);
            while (continuation.isContinue()) {
                nameParser = getPCContext(continuation).p_getNameParser(continuation.getRemainingName(), continuation);
            }
        }
        catch (CannotProceedException cpe) {
            nameParser = NamingManager.getContinuationContext(cpe).getNameParser(cpe.getRemainingName());
        }
        return nameParser;
    }
    
    @Override
    public String composeName(String n, String n2) throws NamingException {
        return this.composeName(new CompositeName(n), new CompositeName(n2)).toString();
    }
    
    @Override
    public Name composeName(Name name, Name name2) throws NamingException {
        Name name3 = (Name)name2.clone();
        if (name == null) {
            return name3;
        }
        name3.addAll(name);
        String s = (String)this.p_getEnvironment().get("java.naming.provider.compose.elideEmpty");
        if (s == null || !s.equalsIgnoreCase("true")) {
            return name3;
        }
        int size = name2.size();
        if (!allEmpty(name2) && !allEmpty(name)) {
            if (name3.get(size - 1).equals("")) {
                name3.remove(size - 1);
            }
            else if (name3.get(size).equals("")) {
                name3.remove(size);
            }
        }
        return name3;
    }
    
    protected static boolean allEmpty(Name name) {
        Enumeration<String> all = name.getAll();
        while (all.hasMoreElements()) {
            if (((String)all.nextElement()).isEmpty()) {
                continue;
            }
            return false;
        }
        return true;
    }
    
    protected static PartialCompositeContext getPCContext(Continuation continuation) throws NamingException {
        Object resolvedObj = continuation.getResolvedObj();
        if ((resolvedObj instanceof PartialCompositeContext)) {
            return (PartialCompositeContext)resolvedObj;
        }
        throw continuation.fillInException(new CannotProceedException());
    }
    
    static {
        _EMPTY_NAME = new CompositeName();
        try {
            PartialCompositeContext._NNS_NAME = new CompositeName("/");
        }
        catch (InvalidNameException ex) {}
    }
}
